@@ -1,10 +1,10 @@
-//! # ndrustfft: *n*-dimensional real-to-complex FFT and real-to-real DCT
+//! # ndrustfft: *n*-dimensional complex-to-complex FFT, real-to-complex FFT and real-to-real DCT
 //!
-//! This library is a wrapper for RustFFT that enables performing FFTs of real-valued data
-//! and DCT's on *n*-dimensional arrays (ndarray).
+//! This library is a wrapper for RustFFT that enables performing FFTs of complex-, real-valued
+//! data and DCT's on *n*-dimensional arrays (ndarray).
 //!
 //! ndrustfft provides Handler structs for FFT's and DCTs, which must be provided
-//! to the respective ndrfft, nddct function alongside with ArrayViews.
+//! to the respective ndrfft, nddct function alongside with Arrays.
 //! The Handlers contain the transform plans and buffers which reduce allocation cost.
 //! The Handlers implement a process function, which is a wrapper around Rustfft's
 //! process function with additional steps, i.e. to provide a real-to complex fft,
@@ -41,7 +41,8 @@
 #![warn(missing_doc_code_examples)]
 extern crate ndarray;
 extern crate rustfft;
-use ndarray::{Array1, ArrayViewMut, Dimension, RemoveAxis, Zip};
+use ndarray::{Array1, ArrayBase, Dimension, RemoveAxis, Zip};
+use ndarray::{Data, DataMut};
 pub use rustfft::num_complex::Complex;
 pub use rustfft::num_traits::Zero;
 pub use rustfft::FftNum;
@@ -57,13 +58,15 @@ macro_rules! create_transform {
         $(#[$meta:meta])* $i: ident, $a: ty, $b: ty, $h: ty, $p: ident
     ) => {
         $(#[$meta])*
-        pub fn $i<T, D>(
-            input: &mut ArrayViewMut<$a, D>,
-            output: &mut ArrayViewMut<$b, D>,
+        pub fn $i<R, S, T, D>(
+            input: &mut ArrayBase<R, D>,
+            output: &mut ArrayBase<S, D>,
             handler: &mut $h,
             axis: usize,
         ) where
             T: FftNum,
+            R: Data<Elem = $a>,
+            S: Data<Elem = $b> + DataMut,
             D: Dimension + RemoveAxis,
         {
             let outer_axis = input.ndim() - 1;
@@ -94,13 +97,15 @@ macro_rules! create_transform {
 macro_rules! create_transform_par {
     ($(#[$meta:meta])* $i: ident, $a: ty, $b: ty, $h: ty, $p: ident) => {
         $(#[$meta])*
-        pub fn $i<T, D>(
-            input: &mut ArrayViewMut<$a, D>,
-            output: &mut ArrayViewMut<$b, D>,
+        pub fn $i<R, S, T, D>(
+            input: &mut ArrayBase<R, D>,
+            output: &mut ArrayBase<S, D>,
             handler: &mut $h,
             axis: usize,
         ) where
             T: FftNum,
+            R: Data<Elem = $a>,
+            S: Data<Elem = $b> + DataMut,
             D: Dimension + RemoveAxis,
         {
             let outer_axis = input.ndim() - 1;
@@ -198,11 +203,33 @@ impl<T: FftNum> FftHandler<T> {
         }
     }
 
+    fn fft_lane(&self, data: &[Complex<T>], out: &mut [Complex<T>]) {
+        self.assert_size(self.n, data.len());
+        self.assert_size(self.n, out.len());
+        for (b, d) in out.iter_mut().zip(data.iter()) {
+            *b = *d;
+        }
+        self.plan_fwd.process(out);
+    }
+
+    fn ifft_lane(&self, data: &[Complex<T>], out: &mut [Complex<T>]) {
+        self.assert_size(self.n, data.len());
+        self.assert_size(self.n, out.len());
+        for (b, d) in out.iter_mut().zip(data.iter()) {
+            *b = *d;
+        }
+        self.plan_bwd.process(out);
+        let n64 = T::from_f64(1. / self.n as f64).unwrap();
+        for b in out.iter_mut() {
+            *b = *b * n64;
+        }
+    }
+
     fn rfft_lane(&mut self, data: &[T], out: &mut [Complex<T>]) {
         self.assert_size(self.n, data.len());
         self.assert_size(self.m, out.len());
         for (b, d) in self.buffer.iter_mut().zip(data.iter()) {
-            *b = Complex::new(*d, T::from_f64(0.0).unwrap());
+            *b = Complex::new(*d, T::zero());
         }
         self.plan_fwd.process(&mut self.buffer);
         for (b, d) in self.buffer[0..self.n / 2 + 1].iter().zip(out.iter_mut()) {
@@ -215,7 +242,7 @@ impl<T: FftNum> FftHandler<T> {
         self.assert_size(self.m, out.len());
         let mut buffer = vec![Complex::zero(); self.n];
         for (b, d) in buffer.iter_mut().zip(data.iter()) {
-            *b = Complex::new(*d, T::from_f64(0.0).unwrap());
+            *b = Complex::new(*d, T::zero());
         }
         self.plan_fwd.process(&mut buffer);
         for (b, d) in buffer[0..self.n / 2 + 1].iter().zip(out.iter_mut()) {
@@ -269,6 +296,55 @@ impl<T: FftNum> FftHandler<T> {
     }
 }
 
+create_transform!(
+    /// Complex-to-complex Fourier Transform (serial).
+    /// # Example
+    /// ```
+    /// use ndarray::{Array, Dim, Ix};
+    /// use ndrustfft::{ndfft, Complex, FftHandler};
+    ///
+    /// let (nx, ny) = (6, 4);
+    /// let mut data = Array::<Complex<f64>, Dim<[Ix; 2]>>::zeros((nx, ny));
+    /// let mut vhat = Array::<Complex<f64>, Dim<[Ix; 2]>>::zeros((nx, ny));
+    /// for (i, v) in data.iter_mut().enumerate() {
+    ///     v.re = i as f64;
+    ///     v.im = -1.0*i as f64;
+    /// }
+    /// let mut handler: FftHandler<f64> = FftHandler::new(ny);
+    /// ndfft(&mut data, &mut vhat, &mut handler, 1);
+    /// ```
+    ndfft,
+    Complex<T>,
+    Complex<T>,
+    FftHandler<T>,
+    fft_lane
+);
+
+create_transform!(
+    /// Complex-to-complex Inverse Fourier Transform (serial).
+    /// # Example
+    /// ```
+    /// use ndarray::{Array, Dim, Ix};
+    /// use ndrustfft::{ndfft, ndifft, Complex, FftHandler};
+    ///
+    /// let (nx, ny) = (6, 4);
+    /// let mut data = Array::<Complex<f64>, Dim<[Ix; 2]>>::zeros((nx, ny));
+    /// let mut vhat = Array::<Complex<f64>, Dim<[Ix; 2]>>::zeros((nx, ny));
+    /// for (i, v) in data.iter_mut().enumerate() {
+    ///     v.re = i as f64;
+    ///     v.im = -1.0*i as f64;
+    /// }
+    /// let mut handler: FftHandler<f64> = FftHandler::new(ny);
+    /// ndfft(&mut data, &mut vhat, &mut handler, 1);
+    /// ndifft(&mut vhat, &mut data, &mut handler, 1);
+    /// ```
+    ndifft,
+    Complex<T>,
+    Complex<T>,
+    FftHandler<T>,
+    ifft_lane
+);
+
 create_transform!(
     /// Real-to-complex Fourier Transform (serial).
     /// # Example
@@ -289,7 +365,13 @@ create_transform!(
     ///     &mut handler,
     ///     0,
     /// );
-    ndrfft, T, Complex<T>, FftHandler<T>, rfft_lane);
+    /// ```
+    ndrfft,
+    T,
+    Complex<T>,
+    FftHandler<T>,
+    rfft_lane
+);
 
 create_transform!(
     /// Complex-to-real inverse Fourier Transform (serial).
@@ -311,18 +393,57 @@ create_transform!(
     ///     &mut handler,
     ///     0,
     /// );
-    ndirfft, Complex<T>, T, FftHandler<T>, irfft_lane);
+    /// ```
+    ndirfft,
+    Complex<T>,
+    T,
+    FftHandler<T>,
+    irfft_lane
+);
+
+create_transform_par!(
+    /// Complex-to-complex Fourier Transform (parallel).
+    ///
+    /// Further infos: see [`ndfft`]
+    ndfft_par,
+    Complex<T>,
+    Complex<T>,
+    FftHandler<T>,
+    fft_lane
+);
+
+create_transform_par!(
+    /// Complex-to-complex inverse Fourier Transform (parallel).
+    ///
+    /// Further infos: see [`ndifft`]
+    ndifft_par,
+    Complex<T>,
+    Complex<T>,
+    FftHandler<T>,
+    ifft_lane
+);
 
 create_transform_par!(
     /// Real-to-complex Fourier Transform (parallel).
     ///
-    /// Further infos: see [`ndirfft`]
-    ndrfft_par, T, Complex<T>, FftHandler<T>, rfft_lane_par);
+    /// Further infos: see [`ndrfft`]
+    ndrfft_par,
+    T,
+    Complex<T>,
+    FftHandler<T>,
+    rfft_lane_par
+);
+
 create_transform_par!(
     /// Complex-to-real inverse Fourier Transform (parallel).
     ///
     /// Further infos: see [`ndirfft`]
-    ndirfft_par, Complex<T>, T, FftHandler<T>, irfft_lane_par);
+    ndirfft_par,
+    Complex<T>,
+    T,
+    FftHandler<T>,
+    irfft_lane_par
+);
 
 /// # *n*-dimensional real-to-real Cosine Transform (DCT-I).
 ///
@@ -399,11 +520,15 @@ impl<T: FftNum> DctHandler<T> {
     /// (a*,b*,c*,d*)
     fn dct1_lane(&mut self, data: &[T], out: &mut [T]) {
         self.assert_size(data.len());
-        let n = self.buffer.len();
-        self.buffer[0] = Complex::new(data[0], T::from_f64(0.0).unwrap());
+        let m = self.buffer.len();
+        for b in self.buffer.iter_mut() {
+            b.re = T::zero();
+            b.im = T::zero();
+        }
+        self.buffer[0] = Complex::new(data[0], T::zero());
         for (i, d) in data[1..].iter().enumerate() {
-            self.buffer[i + 1] = Complex::new(*d, T::from_f64(0.0).unwrap());
-            self.buffer[n - i - 1] = Complex::new(*d, T::from_f64(0.0).unwrap());
+            self.buffer[i + 1] = Complex::new(*d, T::zero());
+            self.buffer[m - i - 1] = Complex::new(*d, T::zero());
         }
         self.plan.process(&mut self.buffer);
         out[0] = self.buffer[0].re;
@@ -414,12 +539,12 @@ impl<T: FftNum> DctHandler<T> {
 
     fn dct1_lane_par(&self, data: &[T], out: &mut [T]) {
         self.assert_size(data.len());
-        let n = self.n;
-        let mut buffer = vec![Complex::zero(); 2 * (self.n - 1)];
-        buffer[0] = Complex::new(data[0], T::from_f64(0.0).unwrap());
+        let m = 2 * (self.n - 1);
+        let mut buffer = vec![Complex::zero(); m];
+        buffer[0] = Complex::new(data[0], T::zero());
         for (i, d) in data[1..].iter().enumerate() {
-            buffer[i + 1] = Complex::new(*d, T::from_f64(0.0).unwrap());
-            buffer[n - i - 1] = Complex::new(*d, T::from_f64(0.0).unwrap());
+            buffer[i + 1] = Complex::new(*d, T::zero());
+            buffer[m - i - 1] = Complex::new(*d, T::zero());
         }
         self.plan.process(&mut buffer);
         out[0] = buffer[0].re;
@@ -460,12 +585,23 @@ create_transform!(
     ///     1,
     /// );
     /// ```
-    nddct1, T, T, DctHandler<T>, dct1_lane);
+    nddct1,
+    T,
+    T,
+    DctHandler<T>,
+    dct1_lane
+);
+
 create_transform!(
     /// Real-to-real Discrete Cosine Transform of type 1 DCT-I  (parallel).
     ///
     /// Further infos: see [`nddct1`]
-    nddct1_par, T, T, DctHandler<T>, dct1_lane_par);
+    nddct1_par,
+    T,
+    T,
+    DctHandler<T>,
+    dct1_lane_par
+);
 
 /// Tests
 #[cfg(test)]
@@ -476,6 +612,56 @@ mod test {
     #[test]
     /// Successive forward and inverse transform
     fn test_fft() {
+        let (nx, ny) = (6, 4);
+        let mut data = Array::<Complex<f64>, Dim<[Ix; 2]>>::zeros((nx, ny));
+        let mut vhat = Array::<Complex<f64>, Dim<[Ix; 2]>>::zeros((nx, ny));
+        for (i, v) in data.iter_mut().enumerate() {
+            v.re = i as f64;
+            v.im = -1.0 * i as f64;
+        }
+        let mut handler: FftHandler<f64> = FftHandler::new(ny);
+        let expected = data.clone();
+        ndfft(&mut data, &mut vhat, &mut handler, 1);
+        ndifft(&mut vhat, &mut data, &mut handler, 1);
+
+        // Assert
+        let dif = 1e-6;
+        for (a, b) in expected.iter().zip(data.iter()) {
+            if (a.re - b.re).abs() > dif {
+                panic!("Large difference of values, got {} expected {}.", b, a)
+            }
+            if (a.im - b.im).abs() > dif {
+                panic!("Large difference of values, got {} expected {}.", b, a)
+            }
+        }
+    }
+
+    #[test]
+    /// Successive forward and inverse transform
+    fn test_dct_serial_vs_parallel() {
+        let (nx, ny) = (6, 4);
+        let mut data = Array::<f64, Dim<[Ix; 2]>>::zeros((nx, ny));
+        let mut vhat = Array::<f64, Dim<[Ix; 2]>>::zeros((nx, ny));
+        let mut vhat_par = Array::<f64, Dim<[Ix; 2]>>::zeros((nx, ny));
+        for (i, v) in data.iter_mut().enumerate() {
+            *v = i as f64;
+        }
+        let mut handler: DctHandler<f64> = DctHandler::new(nx);
+        nddct1(&mut data, &mut vhat, &mut handler, 0);
+        nddct1_par(&mut data, &mut vhat_par, &mut handler, 0);
+
+        // Assert
+        let dif = 1e-6;
+        for (a, b) in vhat.iter().zip(vhat_par.iter()) {
+            if (a - b).abs() > dif {
+                panic!("Large difference of values, got {} expected {}.", b, a)
+            }
+        }
+    }
+
+    #[test]
+    /// Successive forward and inverse transform
+    fn test_rfft() {
         let (nx, ny) = (6, 4);
         let mut data = Array::<f64, Dim<[Ix; 2]>>::zeros((nx, ny));
         let mut vhat = Array::<Complex<f64>, Dim<[Ix; 2]>>::zeros((nx, ny / 2 + 1));
@@ -495,4 +681,35 @@ mod test {
             }
         }
     }
+
+    #[test]
+    /// Successive forward and inverse transform
+    fn test_rfft_serial_vs_parallel() {
+        let (nx, ny) = (6, 4);
+        let mut data = Array::<f64, Dim<[Ix; 2]>>::zeros((nx, ny));
+        let mut vhat = Array::<Complex<f64>, Dim<[Ix; 2]>>::zeros((nx, ny / 2 + 1));
+        let mut vhat_par = Array::<Complex<f64>, Dim<[Ix; 2]>>::zeros((nx, ny / 2 + 1));
+        for (i, v) in data.iter_mut().enumerate() {
+            *v = i as f64;
+        }
+        let mut handler: FftHandler<f64> = FftHandler::new(ny);
+        ndrfft(&mut data.view_mut(), &mut vhat.view_mut(), &mut handler, 1);
+        ndrfft_par(
+            &mut data.view_mut(),
+            &mut vhat_par.view_mut(),
+            &mut handler,
+            1,
+        );
+
+        // Assert
+        let dif = 1e-6;
+        for (a, b) in vhat.iter().zip(vhat_par.iter()) {
+            if (a.re - b.re).abs() > dif {
+                panic!("Large difference of values, got {} expected {}.", b, a)
+            }
+            if (a.im - b.im).abs() > dif {
+                panic!("Large difference of values, got {} expected {}.", b, a)
+            }
+        }
+    }
 }
